"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var fs_1 = require("fs");
var crypto_1 = __importDefault(require("crypto"));
var aws_sdk_1 = __importDefault(require("aws-sdk"));
var AwsConfig = aws_sdk_1.default.config;
var extractSpeechOutputBlocks_1 = __importDefault(require("./internals/utils/extractSpeechOutputBlocks"));
var getSpeechMarksCacheKey = function (speechOutputId) {
    return speechOutputId + ".json";
};
var getAudioCacheKey = function (speechOutputId) { return speechOutputId + ".mp3"; };
var publicPath = "./public/tts/";
var getHash = function (text) {
    return crypto_1.default
        .createHash("md5")
        .update(text)
        .digest("hex");
};
var hasTextChanged = function (speechMarksJson, freshText) {
    var textHashInFile = speechMarksJson.textHash;
    return getHash(freshText) !== textHashInFile;
};
var generateTtsFiles = function (pluginOptions, text, cache, reporter, speechOutputId) { return __awaiter(void 0, void 0, void 0, function () {
    var Polly, ssmlTagsBeforeText, ssmlTagsAfterText, matches, textWithSsmlTags, pollyBaseConfiguration, mp3Data, jsonData, speechMarks, speechMarksJson, json;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                // TODO: move AWS and Polly initialization out of this loop but only initialize if actually some text has changed
                AwsConfig.update(__assign({ region: pluginOptions.awsRegion }, (pluginOptions.awsCredentials && {
                    credentials: {
                        accessKeyId: pluginOptions.awsCredentials.accessKeyId,
                        secretAccessKey: pluginOptions.awsCredentials.secretAccessKey
                    }
                })));
                Polly = new aws_sdk_1.default.Polly({ apiVersion: "2016-06-10" });
                ssmlTagsBeforeText = "";
                ssmlTagsAfterText = "";
                if (pluginOptions.defaultSsmlTags) {
                    if (pluginOptions.defaultSsmlTags.indexOf("$SPEECH_OUTPUT_TEXT") === -1) {
                        throw new Error("If the 'defaultSsmlTags' option is defined it must contain the '$SPEECH_OUTPUT_TEXT' variable (see README file).");
                    }
                    matches = pluginOptions.defaultSsmlTags.match(/(.*)\$SPEECH_OUTPUT_TEXT(.*)/);
                    if (!!matches) {
                        ssmlTagsBeforeText = matches[1];
                        ssmlTagsAfterText = matches[2];
                    }
                    else {
                        throw new Error("Invalid 'defaultSsmlTags' option defined. Check README file for more information about the option.");
                    }
                }
                textWithSsmlTags = "<speak>" + ssmlTagsBeforeText + text + ssmlTagsAfterText + "</speak>";
                pollyBaseConfiguration = {
                    VoiceId: pluginOptions.defaultVoiceId,
                    LexiconNames: pluginOptions.lexiconNames,
                    TextType: "ssml",
                    Text: textWithSsmlTags
                };
                reporter.info("(Re-)generating mp3 for SpeechOutput with ID: " + speechOutputId);
                return [4 /*yield*/, Polly.synthesizeSpeech(__assign({ OutputFormat: "mp3" }, pollyBaseConfiguration)).promise()];
            case 1:
                mp3Data = _a.sent();
                if (mp3Data.AudioStream instanceof Buffer) {
                    cache.cache.set(getAudioCacheKey(speechOutputId), mp3Data.AudioStream);
                }
                reporter.info("(Re-)generating speech marks for SpeechOutput with ID: " + speechOutputId);
                return [4 /*yield*/, Polly.synthesizeSpeech(__assign({ OutputFormat: "json", SpeechMarkTypes: ["word"] }, pollyBaseConfiguration)).promise()];
            case 2:
                jsonData = _a.sent();
                if (jsonData.AudioStream instanceof Buffer) {
                    speechMarks = jsonData.AudioStream.toString();
                    speechMarksJson = JSON.parse("[" + speechMarks.replace(/\}\n\{/g, "},{") + "]");
                    json = {
                        textHash: getHash(text),
                        speechMarks: speechMarksJson
                    };
                    cache.cache.set(getSpeechMarksCacheKey(speechOutputId), json);
                }
                return [2 /*return*/];
        }
    });
}); };
var generateFiles = function (speechOutputBlocks, pluginOptions, cache, reporter) { return __awaiter(void 0, void 0, void 0, function () {
    var i, speechOutputBlock, speechMarks, audio, filesAlreadyExist, eventuallyRegeneratedSpeechMarks, eventuallyRegeneratedAudio;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                i = 0;
                _a.label = 1;
            case 1:
                if (!(i < speechOutputBlocks.length)) return [3 /*break*/, 9];
                speechOutputBlock = speechOutputBlocks[i];
                return [4 /*yield*/, cache.cache.get(getSpeechMarksCacheKey(speechOutputBlock.id))];
            case 2:
                speechMarks = _a.sent();
                return [4 /*yield*/, cache.cache.get(getAudioCacheKey(speechOutputBlock.id))];
            case 3:
                audio = _a.sent();
                filesAlreadyExist = speechMarks && audio;
                if (!(!filesAlreadyExist ||
                    hasTextChanged(speechMarks, speechOutputBlock.text))) return [3 /*break*/, 5];
                return [4 /*yield*/, generateTtsFiles(pluginOptions, speechOutputBlock.text, cache, reporter, speechOutputBlock.id)];
            case 4:
                _a.sent();
                _a.label = 5;
            case 5: return [4 /*yield*/, cache.cache.get(getSpeechMarksCacheKey(speechOutputBlock.id))];
            case 6:
                eventuallyRegeneratedSpeechMarks = _a.sent();
                return [4 /*yield*/, cache.cache.get(getAudioCacheKey(speechOutputBlock.id))];
            case 7:
                eventuallyRegeneratedAudio = _a.sent();
                fs_1.mkdirSync(publicPath, { recursive: true });
                fs_1.writeFileSync(path_1.default.join(publicPath, speechOutputBlock.id + ".mp3"), eventuallyRegeneratedAudio);
                fs_1.writeFileSync(path_1.default.join(publicPath, speechOutputBlock.id + ".json"), JSON.stringify(eventuallyRegeneratedSpeechMarks));
                _a.label = 8;
            case 8:
                i++;
                return [3 /*break*/, 1];
            case 9: return [2 /*return*/];
        }
    });
}); };
module.exports = function (parameters, pluginOptions) { return __awaiter(void 0, void 0, void 0, function () {
    var speechOutputBlocks;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                speechOutputBlocks = extractSpeechOutputBlocks_1.default(parameters.markdownAST, pluginOptions.speechOutputComponentName || "SpeechOutput");
                if (!(speechOutputBlocks.length > 0)) return [3 /*break*/, 2];
                return [4 /*yield*/, generateFiles(speechOutputBlocks, pluginOptions, parameters.cache, parameters.reporter)];
            case 1:
                _a.sent();
                _a.label = 2;
            case 2: return [2 /*return*/, parameters.markdownAST];
        }
    });
}); };
// TODO: make sure if a certain text is no longer existing, related files are deleted as well!
//# sourceMappingURL=index.js.map