"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var deepMap = function (children, map) {
    return react_1.default.Children.map(children, function (child) {
        if (react_1.default.isValidElement(child) && child.props.children) {
            return react_1.default.cloneElement(child, {
                children: deepMap(child.props.children, map)
            });
        }
        return map(child);
    });
};
exports.markChildText = function (children, markedWordIndex) {
    var newChildren = [];
    var currentIndex = 0;
    var markWordIfRequired = function (child) {
        if (!child) {
            return child;
        }
        var isText = typeof child === "string";
        if (isText) {
            if (child.trim().length === 0) {
                return child;
            }
            var isOnlySpecialCharacters = child
                .trim()
                .match(/^[!@#$%^&*(),.?"'`:{}|<>\-_]+$/);
            if (isOnlySpecialCharacters) {
                return child;
            }
            var hasWhiteSpaceBeforeFirstWord = child.match(/^\s/) !== null;
            var hasWhiteSpaceAfterLastWord = child.match(/.*\s$/) !== null;
            var whitespaceOrNewlineRegex = /[\s\n]/;
            var words = child
                .split(whitespaceOrNewlineRegex)
                .filter(function (word) { return word.trim().length > 0; });
            var doesChildContainHighlightedWord = markedWordIndex >= currentIndex &&
                markedWordIndex < currentIndex + words.length;
            if (!doesChildContainHighlightedWord) {
                currentIndex += words.length;
                return child;
            }
            var wordIndexInsideChild = markedWordIndex - currentIndex;
            var textBeforeHighlightedWord = words
                .slice(0, wordIndexInsideChild)
                .join(" ");
            var textAfterHighlightedWord = words
                .slice(wordIndexInsideChild + 1)
                .join(" ");
            var highlightedWordComponent = react_1.default.createElement("mark", null, words[wordIndexInsideChild]);
            currentIndex += words.length;
            return __spreadArrays([
                "" + (hasWhiteSpaceBeforeFirstWord ? " " : "") + (textBeforeHighlightedWord ? textBeforeHighlightedWord + " " : "")
            ], [
                highlightedWordComponent
            ], [
                "" + (textAfterHighlightedWord ? " " + textAfterHighlightedWord : "") + (hasWhiteSpaceAfterLastWord ? " " : "")
            ]);
        }
        return child;
    };
    newChildren.push(deepMap(children, markWordIfRequired));
    return newChildren;
};
var WordMarker = function (props) {
    var _a = react_1.default.useState(props.children), manipulatedChildren = _a[0], setManipulatedChildren = _a[1];
    react_1.default.useEffect(function () {
        setManipulatedChildren(exports.markChildText(props.children, props.markedWordIndex));
    }, [props.children, props.markedWordIndex]);
    return react_1.default.createElement(react_1.default.Fragment, null, manipulatedChildren);
};
exports.default = WordMarker;
//# sourceMappingURL=WordMarker.js.map